BootMod equ     0       ; 0 = Goto, 1 = Sleep
CheckCopy equ   0 
;
;*******************************************************************
; 	ESR_0750.ASM             Stand 01.01.2008
;	Taktgenerator 100 kHz mit 2 Phasen
;*******************************************************************
;
; Dieses Programm erzeugt einen 2 phasen 100 kHz Takt
;
; Taktfrequenz ca. 4.0 MHz mit internem RC-Oszillator
;
; Port-Zuordnung PIC12C508A
;
; GP5: Phase -
; GP4: Phase +
; GP3: Steuereingang
; GP2: Charge Pump
; GP1: Phase -
; GP0: Phase +
;
;*******************************************************************
;
;
; PIC12C508A is the target processor, values decimal, eXpand macros
;
	LIST	p=12C508A,r=dec,x=on
;
        include "P12C508A.INC"
;
#define _INTRC  _IntRC_OSC
;
	__config	_INTRC & _WDT_ON & _CP_OFF & _MCLRE_OFF
;
        __idlocs        0x0750  ; ID ab MPASM V 1.50
;
#define	_reset		0x000   ; eigentlich ja 0x1FFH, dort jedoch
                                ; movlw #RCCALIB
;
; Variablen
;
;
    CBLOCK 0x0A         ; wegen Debugger = 0x0A sonst ab 0x07 moegl.
;
	Delay		; Delay-Counter
        OSCBu1          ; Backup OSCCAL
        OSCBu2          ; Backup OSCCAL
;
    ENDC
;
; I/O-Pins
;
#define Phase3          GPIO,5     ; Phase -
#define Phase2          GPIO,4     ; Phase + 
#define Freeze          GPIO,3     ; 
#define ChargeP         GPIO,2     ; Leitung Ladungspumpe
#define Phase1          GPIO,1     ; Phase -
#define Phase0		GPIO,0     ; Phase +
;
; Register-Bits
;
#define _c              STATUS,C   ; Carry des Statusreg
;
; Baudratendefinitionen:
;
TaktFreq equ 4000000
Takt4    equ TaktFreq/4
DLYCYC	 equ 5

DoDelay macro   NCY
;
; NCY = Number of Cycles (or us at 4 MHz)
;
RCY     set    NCY - DLYCYC
LANZ    set    RCY/3
LOFS    set    RCY - (LANZ*3)
        movlw  LANZ
    if  LOFS == 0
        call    BitDelay0
    endif
    if  LOFS == 1
        call    BitDelay1
    endif
    if  LOFS == 2
        call    BitDelay2
    endif
	endm

Boot_or_Sleep macro
;
; Bootet per Goto oder Sleep je nach BootMod
;
    if BootMod
        SLEEP
    else
        goto    Boot
    endif
;
        endm
;
;*******************************************************************
;
;
;*******************************************************************
; Programmstart + Initialisierung
;*******************************************************************
;
	org	_reset
;
        goto    Visible
OscCopy:
    if CheckCopy == 0
        retlw   0xFF
    else
        retlw   0xFF
    endif
;
Visible:
        xorlw	0xFF
        xorlw	0xFF
        xorlw	0xFF
        xorlw	0xFF
;
; Programmdaten im sichtbaren Bereich
;
VisibleLen set 0x40-Visible
;
    while VisibleLen
        xorlw	0xFF
VisibleLen set VisibleLen - 1
    endw
;
Start:
        movwf   OSCCAL        
        movwf   OSCBu1
        movwf   OSCBu2
;
;Option-Register
;
	movlw   0x98	;timer0 takt intern = Befehlszyklus
        OPTION          ;dadurch PIN5 als Output moeglich
                        ;GP0,1,3 mit Pull-up und Prescaler 1:1 WDT
	movlw	0x15	;Output-Latch in Grundzustand
	movwf	GPIO    ;
	movlw	0x08	;GP3 als input, Rest Output
	tris    GPIO
;
; Delay nach Initialisierung fuer Einschwingen PIN's
;
	DoDelay	500
Loop:
;
; Hauptschleife
;
        CLRWDT
;
        movlw   0x15    ;Output-Latch in Grundzustand
        movwf   GPIO    ; GP2, GP5 = low, rest high
; 1
;
; Check auf Brownout (Variablenverlust)
;
        movf    OSCBu1,W        ; Check auf Oszillator-Werte
        xorwf   OSCBu2,W        ; falls nicht beide gleich
        skpz
        Boot_or_Sleep           ; dann neu Booten
;
	comf	GPIO,F
; 0
;
; Check auf Brownout (Variablenverlust)
;
        movf    OSCBu1,W        ; dto OSCCAL-Register
        xorwf   OSCCAL,W
	nop	
	nop
;
	comf	GPIO,F
; 1
;
        andlw   0xF0
        skpz
        Boot_or_Sleep           ; nicht identisch -> Boot
	nop
;
	comf	GPIO,F		;
; 0
;
        call    OscCopy         ; 4 cy einschl call + Ret
;
	comf	GPIO,F
; 1
        xorwf   OSCCAL,W        ; mit OscCal vergleichen
        andlw   0xF0            ; nur High-Nibble
        skpz
    if CheckCopy == 0
	xorlw	0xFF
    else
        Boot_or_Sleep           ; nicht identisch -> Boot
    endif
;
	comf	GPIO,F
; 0
;       Option-Register auffrischen
;
        movlw   0x98            ; Pull-up on
        OPTION
	nop	
	nop
;
	comf	GPIO,F
; 1
;	Freeze-Pin abfragen
;	
FreezeLoop:
;
	btfss	Freeze
	goto	FreezeLoop
	nop
	nop
;
	comf	GPIO,F
; 0
	goto	Loop		;
;
;
; Unterprogramm Bitende abwarten
;
BitDelay2:
        nop
BitDelay1:
        nop
BitDelay0:
        movwf   Delay
BitDelayL:
        decfsz  Delay,F
        goto    BitDelayL
        return
;
;
    if BootMod
    else
        org     0x1FD
Boot:
        movlw   0x18    ; Reset-Wert
        movwf   STATUS  ; nach Status-Register
    endif
;
        END

